#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

#   Note: when called with "upgrade", "addon-upgrade" or "addon-install" 
#   parameter, the "update" command is always executed first.

. /www/pages/cgi-bin/errlib.sh

export PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin
export SHELL=/bin/sh

LOCKFILE="/var/lock/systemupdate"
LOGDIR="/var/log/dss11-websetup"
mkdir -p $LOGDIR 
UPDATE_CODE="$LOGDIR/last-update-code.txt"
UPDATE_MESSAGE="$LOGDIR/last-update-msg.txt"

UPDATE_CODE_ADDON="$LOGDIR/last-update-code-addon.txt"
UPDATE_MESSAGE_ADDON="$LOGDIR/last-update-msg-addon.txt"

UPDATE_DL_DIR="/var/lib/opkg/download"

signal_led_update() {
    if [[ ! -x /usr/bin/dss11-ledd-client ]]; then
        return 1
    fi

    if [[ "$1" == "start" ]]; then
        /usr/bin/dss11-ledd-client system_update_started
    elif [[ "$1" == "end" ]]; then
        /usr/bin/dss11-ledd-client system_update_completed
    fi
}

if [[ ! -x /usr/bin/opkg ]] || [[ ! -x /usr/bin/addon  ]]; then
    echo "opkg utility or addon wrapper not found" 
    echo "[`date`] command \"$1\" end ---------------" >> "$LOGDIR/update.log"
    echo
    exit $ERR_SETUP 
fi

if [[ -e "$LOCKFILE" ]]; then
    echo "Lock file exists, update is in progress..." 
    echo "[`date`] command \"$1\" end ---------------"
    exit $ERR_DOWAIT 
fi

OPKG_CMD=opkg
# support updates from USB
if [ -x /usr/bin/opkg-usb ] && [ -d /www/pages/local-feed/dss11-usb-upgrade/ipk ]; then
    OPKG_CMD=opkg-usb
fi


if [[ "$1" == "addon-update" ]] || [[ "$1" == "addon-upgrade" ]] ||
   [[ "$1" == "addon-install" ]] || [[ "$1" == "addon-remove" ]]; then
    OPKG_CMD=addon
    UPDATE_CODE="$UPDATE_CODE_ADDON"
    UPDATE_MESSAGE="$UPDATE_MESSAGE_ADDON"
fi

# we are good to go
touch "$LOCKFILE"


if [[ "$1" != "addon-remove" ]]; then
    echo "Executing: $OPKG_CMD update"
    $OPKG_CMD update
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Failed to refresh package information" > $UPDATE_MESSAGE
        echo "Failed to refresh package information, opkg returned: $RETCODE"
        echo "[`date`] command \"$1\" end ---------------"
        echo
        rm -f "$LOCKFILE"
        exit $ERR_OPKG
    fi
fi

if [[ "$1" == "upgrade" ]] || [[ "$1" == "addon-upgrade" ]]; then
    CMD="$1"
    if [[ "$1" == "addon-upgrade" ]]; then
        shift 1
        echo "Executing: $OPKG_CMD upgrade $@"
        $OPKG_CMD upgrade $@
        RETCODE="$?"
    else
        signal_led_update "start"
        if [[ -n "$UPDATE_DL_DIR" ]] && [[ -d "$UPDATE_DL_DIR" ]]; then
            echo "Cleaning download directory $UPDATE_DL_DIR/*"
            rm -f $UPDATE_DL_DIR/*
        else
            echo "Creating download directory $UPDATE_DL_DIR"
            mkdir -p "$UPDATE_DL_DIR"
            RETCODE="$?"
            if [[ "$RETCODE" -ne "0" ]]; then
                echo "Could not create download directory!"
                echo "$ERR_UPDATE" > $UPDATE_CODE
                echo "Could not create download directory" > $UPDATE_MESSAGE
                echo "[`date`] command \"$CMD\" end ---------------"
                echo
                signal_led_update "end"
                rm -f "$LOCKFILE"
                exit $ERR_UPDATE
            fi
        fi

        REQ_SPACE=$(expr `opkg2json --upgrade-size | awk '{ print $9 }'` \* 2)
        AVAILABLE_SPACE=$(expr `df -k | grep root | awk '{ print $4 }'` \* 1024)

        echo "Required space for update: $REQ_SPACE"
        echo "Available free space for update: $AVAILABLE_SPACE"

        if [[ -z "$REQ_SPACE" ]] || [[ -z "$AVAILABLE_SPACE" ]]; then
            echo "Could not calculate free space needed for update!"
            echo "$ERR_UPDATE" > $UPDATE_CODE
            echo "Could not calculate free space needed for update" > $UPDATE_MESSAGE
            echo "[`date`] command \"$CMD\" end ---------------"
            echo
            signal_led_update "end"
            rm -f "$LOCKFILE"
            exit $ERR_UPDATE
        fi

        if [[ $AVAILABLE_SPACE -lt $REQ_SPACE ]]; then
            echo "Not enough free space for update!"
            echo "$ERR_UPDATE" > $UPDATE_CODE
            echo "Not enough free space for update!" > $UPDATE_MESSAGE
            echo "[`date`] command \"$CMD\" end ---------------"
            echo
            signal_led_update "end"
            rm -f "$LOCKFILE"
            exit $ERR_UPDATE
        fi

        cd "$UPDATE_DL_DIR"
        echo "Executing: $OPKG_CMD --download-only upgrade"
        $OPKG_CMD --download-only upgrade
        RETCODE="$?"
        if [[ "$RETCODE" -ne "0" ]]; then
            echo "Error when downloading packages, opkg returned: $RETCODE"
            echo "Cleaning download directory $UPDATE_DL_DIR"
            rm -f $UPDATE_DL_DIR/*
            echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
            echo "Failed to download packages, please try again later!" > $UPDATE_MESSAGE
            echo "[`date`] command \"$CMD\" end ---------------"
            echo
            signal_led_update "end"
            rm -f "$LOCKFILE"
            exit $ERR_OPKG
        fi

        echo "Executing: $OPKG_CMD upgrade --cache=$UPDATE_DL_DIR"
        $OPKG_CMD upgrade --cache=$UPDATE_DL_DIR
        RETCODE="$?"
        echo "Cleaning download directory $UPDATE_DL_DIR/*"
        rm -f $UPDATE_DL_DIR/*
        signal_led_update "end"
    fi

    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Encountered errors during upgrade, please review the log!" > $UPDATE_MESSAGE
        echo "Encountered errors during upgrade, opkg returned: $RETCODE"
        echo "[`date`] command \"$CMD\" end ---------------"
        echo
        rm -f "$LOCKFILE" 
        exit $ERR_UPDATE
    fi
    echo "[`date`] command \"$1\" end ---------------"
    echo
elif [[ "$1" == "addon-install" ]]; then
    shift 1
    echo "Executing: $OPKG_CMD install $@"
    $OPKG_CMD install $@
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Encountered errors during addon install, please review the log!" > $UPDATE_MESSAGE
        echo "Encountered errors during addon install, opkg returned: $RETCODE"
        echo "[`date`] command \"addon-install\" end ---------------"
        echo
        rm -f "$LOCKFILE" 
        exit $ERR_UPDATE
    fi
    echo "[`date`] command \"addon-install\" end ---------------"
    echo
elif [[ "$1" == "addon-remove" ]]; then
    shift 1
    echo "Executing: $OPKG_CMD remove $@"
    $OPKG_CMD remove $@
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        echo "$ERR_OPKG:$RETCODE" > $UPDATE_CODE
        echo "Encountered errors during addon removal, please review the log!" > $UPDATE_MESSAGE
        echo "Encountered errors during addon removal, opkg returned: $RETCODE"
        echo "[`date`] command \"addon-remove\" end ---------------"
        echo
        rm -f "$LOCKFILE"
        exit $ERR_UPDATE
    fi
    echo "[`date`] command \"addon-remove\" end ---------------"
    echo
    
else
    echo "[`date`] command \"$1\" end ---------------"
    echo
fi

rm -f "$LOCKFILE"

echo "$ERR_OK" > $UPDATE_CODE
echo "OK" > $UPDATE_MESSAGE
exit $ERR_OK
